@@ -1,9 +1,9 @@
 // -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; tab-width: 8 -*-
 /* :tabSize=4:indentSize=4:noTabs=false:folding=explicit:collapseFolds=1: */
 //
-// Proxy_Iterator.h: Rcpp R/C++ interface class library -- 
+// Proxy_Iterator.h: Rcpp R/C++ interface class library --
 //
-// Copyright (C) 2010 - 2011 Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2010 - 2013 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -34,41 +34,43 @@ public:
 		typedef int difference_type ;
 		typedef PROXY value_type;
 		typedef std::random_access_iterator_tag iterator_category ;
-		
+
 		Proxy_Iterator( ): proxy(){} ;
 		Proxy_Iterator( const Proxy_Iterator& other) : proxy( other.proxy){}
 		Proxy_Iterator( const PROXY& proxy_ ) : proxy( proxy_ ){} ;
-		
+
 		Proxy_Iterator& operator=( const Proxy_Iterator& other ){
 			proxy.import( other.proxy ) ;
 			return *this ;
 		}
-		
+
 		inline Proxy_Iterator& operator++(){
 			proxy.move(1) ;
 			return *this ;
 		}
-		inline Proxy_Iterator& operator++(int){
-			proxy.move(1) ;
-			return *this ;
+		inline Proxy_Iterator operator++(int){
+			Proxy_Iterator orig(*this) ;
+			++(*this) ;
+			return orig ;
 		}
-		
+
 		inline Proxy_Iterator& operator--(){
 			proxy.move(-1) ;
 			return *this ;
 		}
-		inline Proxy_Iterator& operator--(int){
-			proxy.move(-1) ;
-			return *this ;
+		inline Proxy_Iterator operator--(int){
+			Proxy_Iterator orig(*this) ;
+			--(*this) ;
+			return orig ;
 		}
-		                    
+
 		inline Proxy_Iterator operator+(difference_type n) const {
 			return Proxy_Iterator( PROXY(*proxy.parent, proxy.index + n) ) ;
 		}
 		inline Proxy_Iterator operator-(difference_type n) const {
 			return Proxy_Iterator( PROXY(*proxy.parent, proxy.index - n) ) ;
 		}
-		
+
 		inline Proxy_Iterator& operator+=(difference_type n) {
 			proxy.move( n ) ;
 			return *this ;
@@ -84,7 +86,7 @@ public:
 		inline pointer operator->(){
 			return &proxy ;
 		}
-		
+
 		inline bool operator==( const Proxy_Iterator& y) const {
 			return ( this->proxy.index == y.proxy.index ) && ( this->proxy.parent == y.proxy.parent );
 		}
@@ -103,15 +105,15 @@ public:
 		inline bool operator>=( const Proxy_Iterator& other ) const {
 			return proxy.index >= other.proxy.index ;
 		}
-		
+
 		inline difference_type operator-(const Proxy_Iterator& other) const {
 			return proxy.index - other.proxy.index ;
 		}
 
 		inline int index() const { return proxy.index ; }
 
-		inline PROXY operator[](int i){ return PROXY(*proxy.parent, proxy.index + i) ; } 
-		
+		inline PROXY operator[](int i){ return PROXY(*proxy.parent, proxy.index + i) ; }
+
 private:
 	PROXY proxy ;
 } ;
