@@ -2,7 +2,7 @@
 //
 // RangeIndexer.h: Rcpp R/C++ interface class library --
 //
-// Copyright (C) 2010	Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2010 - 2011 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -23,7 +23,8 @@
 #define Rcpp__vector__RangeIndexer_h
 
 #define UNROLL_LOOP(OP)                              \
-    const T& input = x.get_ref() ;                   \
+    typedef typename ::Rcpp::traits::Extractor<RTYPE,NA,T>::type EXT ; \
+    const EXT& input( x.get_ref() ) ;                   \
     int __trip_count = (size_) >> 2;                 \
     int i=0 ;                                        \
     for ( ; __trip_count > 0 ; --__trip_count) {     \
@@ -45,58 +46,57 @@
     }
 
 
-
+namespace Rcpp{
 namespace internal{
 
-template <int RTYPE, typename VECTOR>
-class RangeIndexer {
+template <int RTYPE, bool NA, typename VECTOR>
+class RangeIndexer : public VectorBase<RTYPE, NA, RangeIndexer<RTYPE,NA,VECTOR> >  {
 public:
 	typedef typename VECTOR::Proxy Proxy ;
 	typedef typename VECTOR::iterator iterator ;
-	
-	// TODO: check if the indexer is valid
-	RangeIndexer( VECTOR& vec_, const Rcpp::Range& range_) : 
+
+	RangeIndexer( VECTOR& vec_, const Rcpp::Range& range_) :
 		start(vec_.begin() + range_.get_start() ), size_( range_.size() ) {}
-	
-		// TODO: size exceptions
-	template <bool NA, typename T>	
-	RangeIndexer& operator=( const Rcpp::VectorBase<RTYPE,NA,T>& x){
+
+	template <bool NA_, typename T>
+	RangeIndexer& operator=( const Rcpp::VectorBase<RTYPE,NA_,T>& x){
 	    UNROLL_LOOP(=)
 	}
-	
-	template <bool NA, typename T>	
-	RangeIndexer& operator+=( const Rcpp::VectorBase<RTYPE,NA,T>& x){
+
+	template <bool NA_, typename T>
+	RangeIndexer& operator+=( const Rcpp::VectorBase<RTYPE,NA_,T>& x){
 		 UNROLL_LOOP(+=)
 	}
-	
-	template <bool NA, typename T>	
-	RangeIndexer& operator*=( const Rcpp::VectorBase<RTYPE,NA,T>& x){
+
+	template <bool NA_, typename T>
+	RangeIndexer& operator*=( const Rcpp::VectorBase<RTYPE,NA_,T>& x){
 		UNROLL_LOOP(*=)
 	}
-	
-	template <bool NA, typename T>	
-	RangeIndexer& operator-=( const Rcpp::VectorBase<RTYPE,NA,T>& x){
+
+	template <bool NA_, typename T>
+	RangeIndexer& operator-=( const Rcpp::VectorBase<RTYPE,NA_,T>& x){
 		UNROLL_LOOP(-=)
 	}
-	
-	template <bool NA, typename T>	
-	RangeIndexer& operator/=( const Rcpp::VectorBase<RTYPE,NA,T>& x){
+
+	template <bool NA_, typename T>
+	RangeIndexer& operator/=( const Rcpp::VectorBase<RTYPE,NA_,T>& x){
 		UNROLL_LOOP(/=)
 	}
-	
-	inline Proxy operator[]( int i ){
+
+	inline Proxy operator[]( int i ) const {
 	    return start[i] ;
 	}
-	
-	inline int size(){
+
+	inline int size() const {
 		return size_ ;
 	}
-	
+
 private:
 	iterator start ;
 	int size_ ;
 } ;
-	
+
+}
 }
 
 #undef UNROLL_LOOP
